import { NestFactory, Reflector } from '@nestjs/core'
import { AppModule } from './app.module'
import { DocumentBuilder, SwaggerModule } from '@nestjs/swagger'
import { ClassSerializerInterceptor, ValidationPipe } from '@nestjs/common'

async function bootstrap() {
  const app = await NestFactory.create(AppModule, { cors: true })

  app.setGlobalPrefix(process.env.API_GLOBAL_PREFIX ?? '')
  app.useGlobalPipes(new ValidationPipe({ transform: true }))
  app.useGlobalInterceptors(new ClassSerializerInterceptor(app.get(Reflector)))

  const { version } = require('../../package.json')
  const config = new DocumentBuilder()
    .setTitle('Traintrape-moi')
    .setDescription('API permettant de stocker les données de Traintrape-moi')
    .setVersion(version)
    .addBearerAuth()
    .build()
  const document = SwaggerModule.createDocument(app, config)

  SwaggerModule.setup(process.env.API_GLOBAL_PREFIX ?? '', app, document)
  await app.listen(process.env.PORT ?? 3000)
}
bootstrap()
